import requests

def chat_yiyan(prompt, his=[], prompt_his_str='你：{}\n分身：{}'):
    from ...local_config import config
    access_token = config['yiyan']['access_token']
    url = f'https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/completions?access_token={access_token}'
    result = requests.post(url, json={
        "messages": [
            {"role": "user", "content": prompt}
        ]
    })
    res_j = result.json()
    if 'result' in res_j:
        return res_j['result']
    if res_j['error_code'] == 18:
        raise Exception('qps limit error')
    print(res_j)
    raise Exception('chat error')

def en2cn_yiyan(prompt):
    Q_motif = f'你是一个有百年经验的英汉翻译官，请你翻译以下句子\n{prompt}\n翻译结果为：'
    result=chat_yiyan(Q_motif)
    return result

def emb_yiyan(txt):
    from ...local_config import config
    if type(txt) == str:
        txts = [txt]
    else:
        txts = txt
    # curl 'https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id=[API Key]&client_secret=[Secret Key]'
    access_token = config['yiyan']['access_token']
    url = f'https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/embeddings/embedding-v1?access_token={access_token}'
    result = requests.post(url, json={
        "input": txts
    })
    vectors = [d['embedding'] for d in result.json()['data']]
    if type(txt) == str:
        return vectors[0]
    return vectors

if __name__ == '__main__':
    print(chat_yiyan('你能做什么'))
    print(en2cn_yiyan('hello world'))
    print(emb_yiyan(['hello world']))



config = {
    'api': 'aistudio', # openai | chatglm_paddle | chatglm | yiyan | xunfei | aistudio
    'en2cn': 'aistudio', # google_trans | chatglm_paddle | yiyan | xunfei | aistudio
    'emb': 'aistudio', # yiyan | xunfei | aistudio
    'log': {
        'silent': True
    },
    'openai': {
        'key': [
            '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',
        ]
    },
    'chatglm': {
        'url': '',
    },
    'yiyan': {
        'access_token': '24.f98fd9187ebb6571194d29108b77a3a9.2592000.1695394520.282335-36387559'
    },
    'xunfei': {
        'appid': '',
        'api_secret': '',
        'api_key': ''
    },
    'aistudio': {
        'access_token': ''
    }
}

import time
import random

def try_more(fn, *args):
    return fn(*args) # 请求单次
    try_times = 0
    while try_times < 20:
        try:
            try_times += 1
            content = fn(*args)
            return content
        except Exception as e:
            if 'qps limit error' in str(e) or 'rate limit' in str(e):
                time.sleep(random.random() * 3)
            else:
                time.sleep(random.random() * 1)
                print(fn, *args, e)
    return ''

# chat
def chat(prompt):
    if config['api'] == 'openai':
        from .utils.api.openai_api import chat_openai as _chat
    elif config['api'] == 'chatglm_paddle':
        from .utils.api.chatglm_paddle import chat_chatglm_paddle as _chat
    elif config['api'] == 'chatglm':
        from .utils.api.chatglm_api import chat_chatglm as _chat
    elif config['api'] == 'yiyan':
        from .utils.api.yiyan_api import chat_yiyan as _chat
    elif config['api'] == 'xunfei':
        from .utils.api.xunfei_api import chat_xunfei as _chat
    elif config['api'] == 'xunfei':
        from .utils.api.xunfei_api import chat_xunfei as _chat
    elif config['api'] == 'aistudio':
        from .utils.api.aistudio_api import chat_aistudio as _chat
    else:
        raise Exception('api not supported')
    return try_more(_chat, prompt)

# en2cn
en_cn_cache = {}
def en2cn(txt, use_cache=True):
    
    if config['en2cn'] == 'google_trans':
        from .utils.api.google_trans import en2cn_google as _en2cn
    elif config['en2cn'] == 'chatglm_paddle':
        from .utils.api.chatglm_paddle import en2cn_glm as _en2cn
    elif config['en2cn'] == 'yiyan':
        from .utils.api.yiyan_api import en2cn_yiyan as _en2cn
    elif config['en2cn'] == 'xunfei':
        from .utils.api.xunfei_api import en2cn_xunfei as _en2cn
    elif config['en2cn'] == 'aistudio':
        from .utils.api.aistudio_api import en2cn_aistudio as _en2cn
    else:
        raise Exception('en2cn not supported')
    if txt.strip() == '':
        return ''
    if txt in en_cn_cache and use_cache:
        return en_cn_cache[txt]
    cn = try_more(_en2cn, txt)
    en_cn_cache[txt] = cn
    return cn

# emb
def emb(txt):
    if config['emb'] == 'yiyan':
        from .utils.api.yiyan_api import emb_yiyan as _emb
    elif config['emb'] == 'xunfei':
        from .utils.api.xunfei_api import emb_xunfei as _emb
    elif config['emb'] == 'aistudio':
        from .utils.api.aistudio_api import emb_aistudio as _emb
    else:
        raise Exception('emb not supported')
    return try_more(_emb, txt)



import sys
from ....utils.format.txt_2_list import txt_2_list
from ....local_config import config
import re

cls_prompt = "你是一个聪明而且有百年经验的文本分类器. 你的任务是从一段文本里面提取出相应的分类结果签。你的回答必须用统一的格式。文本用```符号分割。分类类型保存在一个数组里{类别}" \
        "\n{历史}" \
        "输入|```{原文}```输出|"

def text_classification(src_txt, type_arr, history=[], chat=None, prompt=cls_prompt):
    if not chat:
        from ....local_config import chat
    history_txt = ''.join([f'输入|```{q}```输出|{a}\n' for q, a in history])
    user = prompt
    user = user.replace('{类别}', str(type_arr)).replace('{历史}', history_txt).replace('{原文}', src_txt)
    content = chat(user)
    if not config['log']['silent']:
        print(f'---- text_classification ----\nuser {user}\ncontent {content}\n')
    # 删除非结果部分
    blocks = re.compile(r'[，,.。]').split(content)
    _blocks = []
    for b in blocks:
        is_bad = False
        for not_tip in ['无法', '不属于', '不是']:
            if not_tip in b:
                is_bad = True
                break
        if not is_bad:
            _blocks.append(b)
    content = ''.join(_blocks)
    # Check out in type_arr
    result = []
    for type in type_arr:
        if type in content:
            result.append(type)
            # 删去已经匹配的type
            content = content.replace(type, '')
    return result

if __name__ == '__main__':
    # type_arr = ['好评', '差评']
    # type_arr_txt = "是差评、不是差评"
    # type_arr_txt = "天气查询、股票查询、其他"
    type_arr_txt = '捡起、不捡'
    type_arr = txt_2_list(type_arr_txt)
    txts = [
        # '这个商品真不错',
        # '用着不行',
        # '没用过这么好的东西',
        # '今天天气怎么样',
        '地上有100块钱，但捡了会被抓起来，捡还是不捡起来？',
    ]
    history = [
        # ['这个商品真不错', ['其他']],
    ]
    for txt in txts:
        result = text_classification(txt, type_arr, history)
        print(txt, result)



import requests

# 页面地址 https://fd7fa865d3f27cda69.gradio.live/
# 指定请求的数据
def chat_chatglm(prompt):
  from ...local_config import config
  # 发送POST请求到API
  url = config['chatglm']['url']
  data = {'prompt': prompt}
  response = requests.post(url, json=data)
  # 获取预测结果
  result = response.json()
  print(result)

if __name__ == '__main__':
  content = chat_chatglm('清华大学地址')
  print(content)



import re

def txt_2_list(txt):
  txt = txt.strip()
  if '\n' in txt:
    arr = txt.split('\n')
  else:
    split_token = r'[ ,、，;；《》<>]'
    arr = re.split(split_token, txt)
  rm_token = r'["\'”“‘’。.！!？? 【】\[\]]'
  arr = [re.sub(rm_token, '', item) for item in arr if item != '']
  # 从大到小排序
  arr.sort(key=lambda x: len(x), reverse=True)
  return arr



import json
import sys
import re
from ....local_config import config

ner_prompt = "你是一个聪明而且有百年经验的命名实体识别（NER）识别器. 你的任务是从一段文本里面提取出相应的实体并且给出标签。你的回答必须用统一的格式。文本用```符号分割。输出采用Json的格式并且标记实体在文本中的位置。实体类型保存在一个数组里{类别}\n" \
        '\n输入|```皮卡丘神奇宝贝```输出|[{"name": "皮卡丘", "type": "Person", "start": 0, "end": 3}, {"name": "神奇宝贝", "type": "物种", "start": 4, "end": 8}]' \
        "\n{历史}" \
        "输入|```{原文}```输出|"

def get_ready_key(name, type, start):
    return f'{name}-{type}-{start}'

def extract_named_entities(src_txt, type_arr, history=[], chat=None, prompt=ner_prompt):
    if not chat:
        from ....local_config import chat
    history_txt = ''.join([f'输入|```{q}```输出|{json.dumps(a, ensure_ascii=False)}\n' for q, a in history])
    user = prompt
    user = user.replace('{类别}', str(type_arr)).replace('{历史}', history_txt).replace('{原文}', src_txt)
    content = chat(user)    
    if not config['log']['silent']:
        print(f'---- extract_named_entities ----\nuser {user}\ncontent {content}\n')
    # 跨行替换
    content = re.sub(r'\][^紒]*', ']', content)
    content = re.sub(r'[^紒]*\[', '[', content)
    content = re.sub(r'\\', '', content)
    j = json.loads(content)
    result = []
    j.sort(key=lambda x: x['start']*1000+x['end'])
    ready_keys = set()
    for item in j:
        s = item['start']
        e = item['end']
        # 过滤非目标实体类型
        if not type_arr.__contains__(item['type']):
            continue
        # 修正标注错误的实体坐标
        if src_txt[s:e] != item['name']:
            for i in range(len(src_txt)):
                if src_txt[i:i+len(item['name'])] != item['name']:
                    continue
                # 跳过匹配过的实体，防止重复匹配
                ready_key = get_ready_key(item['name'], item['type'], i)
                if ready_keys.__contains__(ready_key):
                    continue
                item['start'] = i
                break
        # 确保实体结尾坐标正确
        item['end'] = item['start'] + len(item['name'])
        # 将在实体类型里的放入结果
        result.append(item)
        ready_key = get_ready_key(item['name'], item['type'], item['start'])
        ready_keys.add(ready_key)
    return result

if __name__ == '__main__':
    # extract_named_entities("```汤姆每天都被杰瑞欺负，皮卡丘越来越想帮忙，竟然还总是被拒绝，心想难道我“皮大仙”这点能力都没有？而且，这货不是被虐狂吧```", ["Person", "物种"])
    result = extract_named_entities('老百姓心新乡新闻网话说这几天新乡天气还好吧偷笑', ['代称', '行政区'])
    print(result)



import json
from flask import Flask, request, send_file, make_response
from utils.anno.cls.text_classification import text_classification
from utils.anno.ner.entity_extract import extract_named_entities
import time

# Note: Setting CORS to allow chat.openapi.com is only required when running a localhost plugin
# import quart_cors
# app = quart_cors.cors(quart.Quart(__name__), allow_origin="https://chat.openai.com")
# app = quart_cors.cors(quart.Quart(__name__), allow_origin="https://yiyan.baidu.com")
# 因为通过 nginx 走 https 转发，所以不需要设置 CORS，/etc/nginx/nginx.conf 配置如下
# add_header 'Access-Control-Allow-Origin' *;
# location / {
#         proxy_pass http://127.0.0.1:5003;
# }
app = Flask(__name__)

def make_json_response(data, status_code=200):
    response = make_response(json.dumps(data), status_code)
    response.headers["Content-Type"] = "application/json"
    return response

@app.route("/classification/", methods=['POST'])
def classification():
    text = request.json.get('text', '')
    types = request.json.get('types', '')
    time.sleep(3)
    return make_json_response({'message': '123'}, 200)
    result = text_classification(text, types)
    print('result', result)


@app.route("/entityextract/", methods=['POST'])
def entityextract():
    text = request.json.get('text', '')
    types = request.json.get('types', '')
    result = extract_named_entities(text, types)
    return make_json_response({'message': result}, 200)


@app.route("/logo.png")
async def plugin_logo():
    """
        注册用的：返回插件的logo，要求48 x 48大小的png文件.
        注意：API路由是固定的，事先约定的。
    """
    return send_file('logo.png', mimetype='image/png')


@app.route("/.well-known/ai-plugin.json")
async def plugin_manifest():
    """
        注册用的：返回插件的描述文件，描述了插件是什么等信息。
        注意：API路由是固定的，事先约定的。
    """
    host = request.host_url
    with open(".well-known/ai-plugin.json", encoding="utf-8") as f:
        text = f.read().replace("PLUGIN_HOST", host)
        return text, 200, {"Content-Type": "application/json"}


@app.route("/.well-known/openapi.yaml")
async def openapi_spec():
    """
        注册用的：返回插件所依赖的插件服务的API接口描述，参照openapi规范编写。
        注意：API路由是固定的，事先约定的。
    """
    with open(".well-known/openapi.yaml", encoding="utf-8") as f:
        text = f.read()
        return text, 200, {"Content-Type": "text/yaml"}


def main():
    app.run(debug=True, host="0.0.0.0", port=5003)


if __name__ == "__main__":
    main()



# pip install https://studio-package.bj.bcebos.com/aistudio-0.0.2-py3-none-any.whl
import erniebot
erniebot.api_type = 'aistudio'

def chat_aistudio(prompt):
    from ...local_config import config
    erniebot.access_token = config['aistudio']['access_token']
    chat_completion = erniebot.ChatCompletion.create(
        model = 'ernie-bot',
        messages=[{
            "role" : "user" ,
            "content" : prompt
        }]
    )
    content = chat_completion.result
    # print('prompt', prompt)
    # print('content', content)
    return content

def en2cn_aistudio(prompt):
    Q_motif = f'你是一个有百年经验的英汉翻译官，请你翻译以下句子\n{prompt}\n翻译结果为：'
    result=chat_aistudio(Q_motif)
    return result

def emb_aistudio(txt):
    from ...local_config import config
    erniebot.access_token = config['aistudio']['access_token']
    txts = txt
    if type(txt) == str:
        txts = [txt]
    # embeddings max tokens per batch size is 384
    txts = [txt[:384] for txt in txts]
    embeddings = erniebot.Embedding.create(
        model='ernie-text-embedding',
        input=txts,
    )
    if embeddings['error_code'] == 40406:
        raise Exception('qps limit error')
    elif embeddings.error_code:
        raise Exception(embeddings.error_msg)
    vectors = [d['embedding'] for d in embeddings['data']]
    if type(txt) == str:
        return vectors[0]
    return vectors

if __name__ == '__main__':
    print(chat_aistudio("用可爱的语气介绍一下你自己"))
    print(en2cn_aistudio('hello world'))
    print(emb_aistudio(["推荐⼀些美⻝","给我讲个故事"]))



import _thread as thread
import base64
import datetime
import hashlib
import hmac
import json
from urllib.parse import urlparse
import ssl
from datetime import datetime
from time import mktime
import time
from urllib.parse import urlencode
from wsgiref.handlers import format_date_time
import requests

import websocket
import requests

class Ws_Param(object):
    # 初始化
    def __init__(self, APPID, APIKey, APISecret, gpt_url):
        self.APPID = APPID
        self.APIKey = APIKey
        self.APISecret = APISecret
        self.host = urlparse(gpt_url).netloc
        self.path = urlparse(gpt_url).path
        self.gpt_url = gpt_url

    # 生成url
    def create_url(self):
        # 生成RFC1123格式的时间戳
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        # 拼接字符串
        signature_origin = "host: " + self.host + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "GET " + self.path + " HTTP/1.1"

        # 进行hmac-sha256进行加密
        signature_sha = hmac.new(self.APISecret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()

        signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding='utf-8')

        authorization_origin = f'api_key="{self.APIKey}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature_sha_base64}"'

        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')

        # 将请求的鉴权参数组合为字典
        v = {
            "authorization": authorization,
            "date": date,
            "host": self.host
        }
        # 拼接鉴权参数，生成url
        url = self.gpt_url + '?' + urlencode(v)
        # 此处打印出建立连接时候的url,参考本demo的时候可取消上方打印的注释，比对相同参数时生成的url与自己代码生成的url是否一致
        return url


# 收到websocket错误的处理
def on_error(ws, error):
    print("### error:", error)


# 收到websocket关闭的处理
def on_close(ws):
    # print("### closed ###")
    pass


# 收到websocket连接建立的处理
def on_open(ws):
    thread.start_new_thread(run, (ws,))


def run(ws, *args):
    data = json.dumps(gen_params(appid=ws.appid, question=ws.question))
    ws.send(data)


# 收到websocket消息的处理
def on_message(ws, message):
    # print(message)
    data = json.loads(message)
    code = data['header']['code']
    if code != 0:
        print(f'请求错误: {code}, {data}')
        ws.close()
        return 404, ''
    else:
        choices = data["payload"]["choices"]
        status = choices["status"]
        content = choices["text"][0]["content"]
        # print(content, end='')
        if status == 2:
            ws.close()
            return 200, content
    return 0, content


def gen_params(appid, question):
    """
    通过appid和用户的提问来生成请参数
    """
    data = {
        "header": {
            "app_id": appid,
            "uid": "1234"
        },
        "parameter": {
            "chat": {
                "domain": "general",
                "random_threshold": 0.5,
                "max_tokens": 2048,
                "auditing": "default"
            }
        },
        "payload": {
            "message": {
                "text": [
                    {"role": "user", "content": question}
                ]
            }
        }
    }
    return data


def main(appid, api_key, api_secret, gpt_url, question):
    outs = []
    state = {'is_close': False}
    cnt = 0
    def _on_message(ws, message):
        is_close, content = on_message(ws, message)
        state['is_close'] = is_close
        outs.append(content)
    def _on_error(ws, error):
        on_error(ws, error)
        state["is_close"] = True
    def _on_close(ws, *args):
        on_close(ws)
        state["is_close"] = True
    wsParam = Ws_Param(appid, api_key, api_secret, gpt_url)
    websocket.enableTrace(False)
    wsUrl = wsParam.create_url()
    ws = websocket.WebSocketApp(wsUrl, on_message=_on_message, on_error=_on_error, on_close=_on_close, on_open=on_open)
    ws.appid = appid
    ws.question = question
    ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
    while cnt < 5*30: # 最多等30秒
        time.sleep(0.2)
        if state['is_close']:
            return ''.join(outs)
    return ''
def chat_xunfei(prompt):
    from ...local_config import config
    xunfei_api = config['xunfei']
    content = main(appid=xunfei_api['appid'],
         api_secret=xunfei_api["api_secret"],
         api_key=xunfei_api["api_key"],
         gpt_url="ws://spark-api.xf-yun.com/v1.1/chat",
         question=prompt)
    return content

def en2cn_xunfei(prompt):
    Q_motif = f'你是一个有百年经验的英汉翻译官，请你翻译以下句子\n{prompt}\n翻译结果为：'
    result=chat_xunfei(Q_motif)
    return result

class EmbeddingReq(object):

    def __init__(self, appid, api_key, api_secret, embedding_url):
        self.APPID = appid
        self.APIKey = api_key
        self.APISecret = api_secret
        self.host = urlparse(embedding_url).netloc
        self.path = urlparse(embedding_url).path
        self.url = embedding_url

    # 生成url
    def create_url(self):
        # 生成RFC1123格式的时间戳
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        # 拼接字符串
        signature_origin = "host: " + self.host + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "POST " + self.path + " HTTP/1.1"

        # 进行hmac-sha256进行加密
        signature_sha = hmac.new(self.APISecret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()

        signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding='utf-8')

        authorization_origin = f'api_key="{self.APIKey}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature_sha_base64}"'

        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')

        # 将请求的鉴权参数组合为字典
        v = {
            "authorization": authorization,
            "date": date,
            "host": self.host
        }
        # 拼接鉴权参数，生成url
        url = self.url + '?' + urlencode(v)
        # 此处打印出建立连接时候的url,参考本demo的时候可取消上方打印的注释，比对相同参数时生成的url与自己代码生成的url是否一致
        return url

    def get_Embedding(self, text):
        param_dict = {
            'header': {
                'app_id': self.APPID
            },
            'payload': {
                'text': text
            }
        }
        response = requests.post(url=self.create_url(), json=param_dict)
        result = json.loads(response.content.decode('utf-8'))
        if result['header']['code'] == 0:
            return result['payload']['text']['vector']
        return None

def emb_xunfei(prompt):
    from ...local_config import config
    xunfei_api = config['xunfei']
    embedding_xunfei = EmbeddingReq(
            appid=xunfei_api['appid'],
            api_key=xunfei_api['api_key'],
            api_secret=xunfei_api['api_secret'],
            embedding_url="https://knowledge-retrieval.cn-huabei-1.xf-yun.com/v1/aiui/embedding/query"
        )
    emb = embedding_xunfei.get_Embedding(prompt)
    return emb

"""
    1、配置好python、pip的环境变量
    2、执行 pip install websocket 与 pip3 install websocket-client
    3、去控制台https://console.xfyun.cn/services/cbm获取appid等信息填写即可
"""
if __name__ == "__main__":
    # 测试时候在此处正确填写相关信息即可运行
    print(chat_xunfei("你是谁？你能做什么"))
    print(emb_xunfei("这个问题的向量是什么？"))



import json
import re
from tqdm import tqdm

input_set = set()
type_set = set()

def strip_csv(src):
    if src and src[0] == '"' and src[-1] == '"':
        return src[1:-1]
    return src

def get_j(j):
    annos = [row['value']['choices'][0] for row in j['annotations'][0]['result']]
    input = j['data']['text']
    return annos, input

# ls = open(fn, 'r', encoding='utf-8').read().strip().split('\n')
# ls = ls[1:]
def json_2_json(fn, trans_fun=None, ls_fun=None):
    src = open(fn, 'r', encoding='utf-8').read().strip()
    if ls_fun:
        ls = ls_fun(src)
    elif src[0] == '[':
        ls = json.loads(src)
    else:
        ls = src.split('\n')
    fh = open(re.sub(r'\..*', '.aa.jsonl', fn), 'w', encoding='utf-8')
    out_js = []
    for l in tqdm(ls):
        j = l
        if l and type(l) == str and l[0] == '{':
            j = json.loads(l)
        out_j = {
            'types': trans_fun['types'](j),
            'annos': trans_fun['annos'](j),
            'input': trans_fun['input'](j),
        }
        out_js.append(out_j)
        fh.write(json.dumps(out_j, ensure_ascii=False) + '\n')
    return out_js

if __name__ == '__main__':
    import random
    import pandas as pd
    
    # fn = 'E:/NLP/NER/54_cls_ChnSentiCorp_htl_all.csv'
    # types = ['负面', '正面']
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: types[j[0]],
    #     'input': lambda j: j[1],
    # }, ls_fun=lambda src: [[int(l[0]), strip_csv(l[2:])] for l in src.strip().split('\n')[1:]])
    # fn = 'E:/NLP/NER/01_cls_label_studio.json'
    # types = '病情诊断\n治疗方案\n病因分析\n指标解读\n就医建议\n疾病表述\n后果表述\n注意事项\n功效作用\n医疗费用\n其他'.split('\n')
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: [row['value']['choices'][0] for row in j['annotations'][0]['result']],
    #     'input': lambda j: j['data']['text'],
    # })
    # fn = 'E:/NLP/NER/51_cls_toutiao_cat_data.txt'
    # code_type_map = { '100': '民生', '101': '文化', '102': '娱乐', '103': '体育', '104': '财经', '106': '房产', '107': '汽车', '108': '教育', '109': '科技', '110': '军事', '112': '旅游', '113': '国际', '114': '证券', '115': '农业', '116': '电竞', }
    # types = list(code_type_map.values())
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: [code_type_map[j[1]]],
    #     'input': lambda j: j[-2],
    # }, ls_fun=lambda src: [l.split('_!_') for l in src.strip().split('\n')])
    # fn = 'E:/NLP/NER/44_cls_guwenee.json'
    # types = ['人生','战争','政治','民事','日常','文化','地理']
    # json_2_json(fn, trans_fun={
    #     'types': lambda j: types,
    #     'annos': lambda j: list(set([event['class'] for event in j['event_list']])),
    #     'input': lambda j: j['text'],
    # })
    # fn = 'E:/NLP/NER/20_cls_fenlei_odkg_document.json'
    # types = ['原材料工业管理', '辐射源安全监管', '电子基础产品行业管理工作', '交通', '商贸', '科技管理', '纺织行业管理', '产业政策', '科技管理,高技术管理', '宗教', '水文', '规划', '质量管理工作', '卫生', '消费品工业管理', '司法', '消费电子行业管理工作', '环境标准', '救灾', '地震', '装备工业管理', '技术基础管理', '人口与计划生育', '技术创新管理', '旅游', '消费品工业综合指导工作', '林业', '装备工业管理,其他', '综合政务', '测绘', '政策法规', '人事教育', '民族', '核设施安全监管', '科技', '无线电管理', '航空', '人事', '公安', '工业', '能源', '财经政策', '运行监测协调', '电子信息', '有色金属工业管理', '食品工业行业管理', '教育', '软件和信息技术', '畜牧业', '装备工业行业管理', '渔业', '安全', '国防', '环境保护', '标准', '执法监督', '通信发展', '原材料工业管理,石化化工行业管理', '扶贫', '汽车工业行业管理', '城乡建设', '原材料工业管理,钢铁', '水利', '监察', '妇女儿童工作', '规划,装备工业管理', '信息通信管理', '核电厂安全监管', '其他生态环境管理业务信息', '综合协调管理,其他', '科技管理,产业政策,规划', '农业', '其他', '对外事务', '网络安全管理', '劳动', '国土资源', '节能与综合利用', '民政', '集成电路与半导体器件产业管理工作', '中小企业管理', '海关', '体育', '外事', '安全生产', '核与辐射安全管理']
    # json_2_json(fn, trans_fun={
    #     # 'types': lambda j: [types.add(type.split('\\')[0]) for type in j['分类'].split('、')],
    #     'types': lambda j: list(set(sum([random.choices(types, k=5), [type.split('\\')[0] for type in j['分类'].split('、')] if j['分类'] else []], []))),
    #     'annos': lambda j: [type.split('\\')[0] for type in j['分类'].split('、')] if j['分类'] else [],
    #     'input': lambda j: j['正文'],
    # })
    # fn = 'E:/NLP/NER/20_cls_tygwlb_odkg_document.json'
    # types = ['通告', '通报', '命令', '报告', '公告', '通知', '决定', '函', '意见', '批复']
    # json_2_json(fn, trans_fun={
    #     # 'types': lambda j: [types.append(type) for type in j['通用公文类别'].split('、')],
    #     'types': lambda j: types,
    #     'annos': lambda j: [j['通用公文类别']],
    #     'input': lambda j: j['正文'],
    # })
    # fn = 'E:/NLP/NER/20_cls_ywgwlb_odkg_document.json'
    # types = ['科技-国防科技', '军事-人事任命', '安全-网络安全', '军事-军事批复', '科技-创新改革', '安全-核安全', '军事-战功命令', '科技-航空探月', '科技-新能源', '安全-社会安全', '政府类型公文', '军事-征兵通知']
    # json_2_json(fn, trans_fun={
    #     # 'types': lambda j: [types.append(type) for type in j['业务公文类别'].split('、')],
    #     'types': lambda j: types,
    #     'annos': lambda j: [j['业务公文类别']],
    #     'input': lambda j: j['标题'],
    # })
    # print(list(set(types)))
    # fn = 'E:/NLP/NER/32_cls_event_type_entity_extract_train.csv'
    # types = ['涉嫌传销', '资金账户风险', '财务造假', '失联跑路', '涉嫌欺诈', '重组失败', '资产负面', '产品违规', '业绩下滑', '不能履职', '歇业停业', '公司股市异常', '信批违规', '高管负面', '提现困难', '其他', '实控人股东变更', '投诉维权', '涉嫌违法', '涉嫌非法集资', '评级调整', '交易违规']
    # out_js = json_2_json(fn, trans_fun={
    #     'types': lambda j: list(set(sum([types[:5], [j[-2]]], []))),
    #     'annos': lambda j: [j[-2]],
    #     'input': lambda j: j[1],
    # }, ls_fun=lambda src: [l[1:-1].split('","') for l in src.split('\n')])
    # fn = 'E:/NLP/NER/26_cls_pokemons.csv'
    # types = ['岩石', '飞行', '龙', '电', '一般', '恶', '火', '幽灵', '水', '钢', '地面', '草', '格斗', '虫', '超能力', '妖精', '冰', '毒']
    # out_js = json_2_json(fn, trans_fun={
    #     'types': lambda j: list(set(sum([random.choices(types, k=5), j[0].split('|[')[0].split('|')], []))),
    #     'annos': lambda j: j[0].split('|[')[0].split('|'),
    #     'input': lambda j: j[-1],
    # }, ls_fun=lambda src: [j for j in sum([
    #     [[j[5], j[-7]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[5], j[-6]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[5], j[-5]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[5], j[-4]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[5], j[-3]] for j in pd.read_csv(fn).values.tolist()[1:]],
    # ], []) if str(j[-1]) != 'nan'])
    # types = list(set(sum([j['types'] for j in out_js], [])))
    # fn = 'E:/NLP/NER/26_cls_color_pokemons.csv'
    # types = ['褐色', '黄色', '粉红色', '黑色', '绿色', '紫色', '未知色', '红色', '灰色', '白色', '蓝色']
    # out_js = json_2_json(fn, trans_fun={
    #     # 'types': lambda j: [j[0]],
    #     'types': lambda j: list(set(sum([random.choices(types, k=5), [j[0]]], []))),
    #     'annos': lambda j: [j[0]],
    #     'input': lambda j: j[-1],
    # }, ls_fun=lambda src: [j for j in sum([
    #     [[j[10], j[-7]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[10], j[-6]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[10], j[-5]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[10], j[-4]] for j in pd.read_csv(fn).values.tolist()[1:]],
    #     [[j[10], j[-3]] for j in pd.read_csv(fn).values.tolist()[1:]],
    # ], []) if str(j[-1]) != 'nan'])
    # types = list(set(sum([j['types'] for j in out_js], [])))
    fn = 'E:/NLP/NER/26_cls_class_pokemons.csv'
    # types = []
    types = ['单朵宝可梦', '陆鲨宝可梦', '修行宝可梦', '蓄电池宝可梦', '雪笠宝可梦', '看护宝可梦', '南海宝可梦', '维京宝可梦', '王者宝可梦', '巨大宝可梦', '恶颜宝可梦', '小猪宝可梦', '猪猴宝可梦', '火鼠宝可梦', '王剑宝可梦', '鸟宝可梦', '勾蜓宝可梦', '昆虫宝可梦', '大胃王宝可梦', '大角宝可梦', '双面宝可梦', '境界宝可梦', '新雪宝可梦', '漩涡宝可梦', '叶子宝可梦', '狐狸宝可梦', '波纹宝可梦', '幼君宝可梦', '鼠宝可梦', '脑宝可梦', '时空穿越宝可梦', '水兔宝可梦', '季节宝可梦', '不倒翁宝可梦', '发电宝可梦', '火炬宝可梦', '巨蜈蚣宝可梦', '突击宝可梦', '半梦半醒宝可梦', '音波宝可梦', '清高宝可梦', '骑兵宝可梦', '苹果翅宝可梦', '幼蜂宝可梦', '贪欲宝可梦', '挖洞宝可梦', 'ＤＮＡ宝可梦', '毒针宝可梦', '潜水宝可梦', '前锋宝可梦', '拳法宝可梦', '魔法宝可梦', '岩石肤宝可梦', '怨念宝可梦', '捉弄宝可梦', '油灯宝可梦', '反抗宝可梦', '喷粉宝可梦', '大声宝可梦', '重量级宝可梦', '冥想宝可梦', '入门宝可梦', '天线罩宝可梦', '长颈宝可梦', '进化宝可梦', '罗盘宝可梦', '河狸宝可梦', '石墙宝可梦', '古虾宝可梦', '牙蛇宝可梦', '搬运宝可梦', '虫宝宝宝可梦', '冰蛾宝可梦', '灯饰宝可梦', '抓狂宝可梦', '软体生物宝可梦', '耐力宝可梦', '毒蜂宝可梦', '远古鸟宝可梦', '月光宝可梦', '垃圾场宝可梦', '听力宝可梦', '鳞粉宝可梦', '锹形虫宝可梦', '白鸟宝可梦', '火山宝可梦', '海底宝可梦', '地蛇宝可梦', '刺铠宝可梦', '无限宝可梦', '芳草宝可梦', '爽朗宝可梦', '自制宝可梦', '蝴蝶宝可梦', '球鲸宝可梦', '水枪宝可梦', '气体状宝可梦', '化石宝可梦', '荆棘宝可梦', '铁球宝可梦', '合金宝可梦', '拍手宝可梦', '冰块宝可梦', '风隐宝可梦', '放水宝可梦', '陨石宝可梦', '雪国宝可梦', '风筝宝可梦', '放电宝可梦', '利刃宝可梦', '天空宝可梦', '磁场宝可梦', '流氓宝可梦', '雷宝可梦', '哼唱宝可梦', '蝎怪宝可梦', '橡实宝可梦', '铳炮宝可梦', '围巾宝可梦', '地幔宝可梦', '水珠宝可梦', '水母宝可梦', '摔角宝可梦', '新月宝可梦', '沙蛇宝可梦', '独唱者宝可梦', '蜂巢宝可梦', '发酵宝可梦', '海鼬宝可梦', '闪电宝可梦', '暴牛宝可梦', '树冰宝可梦', '野猪宝可梦', '铁爪宝可梦', '石居宝可梦', '喷火宝可梦', '深海宝可梦', '噪音宝可梦', '钻锥宝可梦', '蜈蚣宝可梦', '子弹宝可梦', '祝福宝可梦', '小熊宝可梦', '穿刺宝可梦', '大火猪宝可梦', '空壳宝可梦', '孤独宝可梦', '影子宝可梦', '雏鸡宝可梦', '蓑衣蛾宝可梦', '筋骨宝可梦', '冰雪宝可梦', '青铜宝可梦', '凶恶宝可梦', '蚁狮宝可梦', '蜷缩宝可梦', '水鸟宝可梦', '野鸽宝可梦', '胃袋宝可梦', '小鱼宝可梦', '镰草宝可梦', '人工宝可梦', '泡沫奶油宝可梦', '黑金宝可梦', '飞蝎宝可梦', '雏鹰宝可梦', '棱镜宝可梦', '破冰宝可梦', '幼狮宝可梦', '鼹鼠宝可梦', '沙丘宝可梦', '海百合宝可梦', '怕麻烦宝可梦', '象征宝可梦', '柔术宝可梦', '护盾宝可梦', '红茶宝可梦', '五星宝可梦', '蟋蟀宝可梦', '国王宝可梦', '螳螂宝可梦', '分割宝可梦', '毒蛾宝可梦', '河马宝可梦', '宁静宝可梦', '停止宝可梦', '盆栽宝可梦', '神秘宝可梦', '怪力宝可梦', '毒蜥宝可梦', '白阳宝可梦', '大头目宝可梦', '心情宝可梦', '龙玉宝可梦', '藤壶宝可梦', '知更鸟宝可梦', '三头鸟宝可梦', '顽皮宝可梦', '热气球宝可梦', '雷击宝可梦', '箭羽宝可梦', '蜕皮宝可梦', '重量宝可梦', '污泥宝可梦', '放哨宝可梦', '连结宝可梦', '电气宝可梦', '暹罗猫宝可梦', '凶暴宝可梦', '萤火虫宝可梦', '独角宝可梦', '海参宝可梦', '烈马宝可梦', '新绿宝可梦', '冻结宝可梦', '甲胄宝可梦', '破坏宝可梦', '电击宝可梦', '强臂宝可梦', '蜥蜴宝可梦', '乌鸦宝可梦', '冰冻宝可梦', '鱼宝可梦', '杂草宝可梦', '齿轮宝可梦', '忍者宝可梦', '鸭宝可梦', '节拍宝可梦', '像铜宝可梦', '鸟嘴宝可梦', '舔舔宝可梦', '顽童宝可梦', '铃铛宝可梦', '煤炭宝可梦', '气球宝可梦', '驴宝可梦', '引诱宝可梦', '泼水宝可梦', '绵羊宝可梦', '沼鱼宝可梦', '棉球宝可梦', '迟钝宝可梦', '鼓手宝可梦', '猫头鹰宝可梦', '树丛宝可梦', '刃羽宝可梦', '企鹅宝可梦', '感谢宝可梦', '针球宝可梦', '空手道宝可梦', '毒气宝可梦', '头锤宝可梦', '浮鲸宝可梦', '鲜奶油宝可梦', '牙宝可梦', '月轮宝可梦', '电鱼宝可梦', '催眠宝可梦', '亲子宝可梦', '仙人掌宝可梦', '捕蝇宝可梦', '蹲守宝可梦', '捕虫宝可梦', '育儿宝可梦', '人形宝可梦', '细胞宝可梦', '海鸥宝可梦', '金鱼宝可梦', '刀刃宝可梦', '狗宝可梦', '水果宝可梦', '猛推宝可梦', '大陆宝可梦', '暗黑宝可梦', '回转宝可梦', '冰山宝可梦', '灾祸宝可梦', '渡魂宝可梦', '回游宝可梦', '懒人宝可梦', '海兔宝可梦', '威吓宝可梦', '爆燃宝可梦', '苹果居宝可梦', '树桩宝可梦', '黑阴宝可梦', '缠人宝可梦', '勇猛宝可梦', '矿石宝可梦', '倒立宝可梦', '细语宝可梦', '电光宝可梦', '知识宝可梦', '须鱼宝可梦', '古代龟宝可梦', '发热宝可梦', '水黾宝可梦', '寂静宝可梦', '一口吞宝可梦', '膨胀宝可梦', '猛火宝可梦', '幸福宝可梦', '蜗牛宝可梦', '海胆宝可梦', '樱花宝可梦', '小猴宝可梦', '铁心宝可梦', '巡回宝可梦', '冻原宝可梦', '念力宝可梦', '拳击宝可梦', '带电宝可梦', '双手宝可梦', '强者宝可梦', '坏心眼宝可梦', '旋风宝可梦', '长鼻宝可梦', '保姆宝可梦', '洞穴宝可梦', '皇帝宝可梦', '似草宝可梦', '风铃宝可梦', '花宝可梦', '爱骨宝可梦', '刺猴宝可梦', '火猫宝可梦', '天体宝可梦', '吐泡宝可梦', '结晶宝可梦', '碎藻宝可梦', '暴风雪宝可梦', '凝视宝可梦', '雷电宝可梦', '波导宝可梦', '打斗宝可梦', '新种宝可梦', '隐形宝可梦', '虎斑猫宝可梦', '宝石宝可梦', '爆炎宝可梦', '钩爪宝可梦', '尿布宝可梦', '草原宝可梦', '小狗宝可梦', '哀怨宝可梦', '亲吻宝可梦', '日轮宝可梦', '发射宝可梦', '邪恶宝可梦', '椋鸟宝可梦', '阴暗宝可梦', '求爱宝可梦', '贤者宝可梦', '刺球宝可梦', '椰子宝可梦', '幼虫宝可梦', '音符宝可梦', '偶像宝可梦', '震动宝可梦', '熔岩宝可梦', '水泡宝可梦', '警戒宝可梦', '贪吃宝可梦', '婴儿宝可梦', '岩窟宝可梦', '茧蜈蚣宝可梦', '欺骗宝可梦', '喷射宝可梦', '刀剑宝可梦', '庞克宝可梦', '阵形宝可梦', '小猫宝可梦', '花饰宝可梦', '绵毛宝可梦', '夜啼宝可梦', '瞌睡宝可梦', '藤蔓状宝可梦', '忠犬宝可梦', '蘑菇宝可梦', '特工宝可梦', '铁蚁宝可梦', '沙堡宝可梦', '挽马宝可梦', '火种宝可梦', '天气宝可梦', '吐火宝可梦', '恶党宝可梦', '芳香宝可梦', '暴君宝可梦', '蒸汽宝可梦', '南瓜宝可梦', '绵鸟宝可梦', '脸面宝可梦', '钳子宝可梦', '花束宝可梦', '喜剧演员宝可梦', '电蜘蛛宝可梦', '刺栗宝可梦', '燕子宝可梦', '星形宝可梦', '密林宝可梦', '高压宝可梦', '花镰宝可梦', '操纵宝可梦', '两头鸟宝可梦', '旋律宝可梦', '啃咬宝可梦', '幼鸡宝可梦', '狼宝可梦', '甲壳宝可梦', '屏障宝可梦', '电松鼠宝可梦', '头锤牛宝可梦', '灯宝可梦', '牙蝎宝可梦', '草猴宝可梦', '猛冲宝可梦', '粗暴宝可梦', '刺果宝可梦', '嫩叶宝可梦', '流星宝可梦', '土地神宝可梦', '恶猿宝可梦', '星云宝可梦', '滚球宝可梦', '家家酒宝可梦', '变色宝可梦', '感情宝可梦', '似鸟宝可梦', '咬住宝可梦', '烟火宝可梦', '毛毛虫宝可梦', '水蜥宝可梦', '垃圾袋宝可梦', '啄木鸟宝可梦', '包容宝可梦', '水鼠宝可梦', '蜂虻宝可梦', '画画宝可梦', '创造宝可梦', '自尊心宝可梦', '海狮宝可梦', '肃静宝可梦', '七星宝可梦', '泥偶宝可梦', '岩石宝可梦', '原始星宝可梦', '抑制宝可梦', '种子宝可梦', '恶狐宝可梦', '黄嘴鸭宝可梦', '岩山宝可梦', '珊瑚宝可梦', '爆炸龟宝可梦', '圆鼠宝可梦', '栖影宝可梦', '踢腿宝可梦', '非星宝可梦', '喇叭喙宝可梦', '地底宝可梦', '坚硬脑袋宝可梦', '烈火宝可梦', '长尾宝可梦', '太阳宝可梦', '食梦宝可梦', '配合宝可梦', '装乖宝可梦', '蝙蝠宝可梦', '超极巨宝可梦', '钢甲鸟宝可梦', '黑暗宝可梦', '奶牛宝可梦', '铁蛇宝可梦', '小鼠宝可梦', '布偶宝可梦', '模仿宝可梦', '增幅宝可梦', '双牙宝可梦', '结冰宝可梦', '坐骑宝可梦', '无赖宝可梦', '磨牙宝可梦', '生命宝可梦', '贵宾犬宝可梦', '健美宝可梦', '水鱼宝可梦', '火山口宝可梦', '人偶宝可梦', '加油宝可梦', '憨憨宝可梦', '发光宝可梦', '发射器宝可梦', '栗鼠宝可梦', '猛禽宝可梦', '长腿宝可梦', '幼燕宝可梦', '棺木宝可梦', '龟宝可梦', '小龟宝可梦', '毛蟹宝可梦', '利器宝可梦', '巨石宝可梦', '人造宝可梦', '招手宝可梦', '火猪宝可梦', '虹色宝可梦', '倒转宝可梦', '香水宝可梦', '丰饶宝可梦', '乘载宝可梦', '清水蟹宝可梦', '猫鼬宝可梦', '音速宝可梦', '海獭宝可梦', '双壳贝宝可梦', '魔像宝可梦', '默剧宝可梦', '长寿宝可梦', '秩序宝可梦', '沙漠鳄鱼宝可梦', '闪光宝可梦', '魂宝可梦', '祈愿宝可梦', '毒袋宝可梦', '吸附宝可梦', '颚斧宝可梦', '集合宝可梦', '武术宝可梦', '裁缝宝可梦', '大颚宝可梦', '棉草宝可梦', '寄居蟹宝可梦', '天雷宝可梦', '等离子宝可梦', '根茎宝可梦', '薄翼宝可梦', '宽大宝可梦', '浮萍宝可梦', '极光宝可梦', '疾行宝可梦', '臭鼬宝可梦', '火宝可梦', '忍耐宝可梦', '陷阱宝可梦', '鳞片宝可梦', '花苞宝可梦', '豆狸宝可梦', '蓑衣虫宝可梦', '足不出叶宝可梦', '钥匙串宝可梦', '草蛇宝可梦', '弹跳宝可梦', '寄生宝可梦', '基因宝可梦', '霓虹宝可梦', '凶猛宝可梦', '脱壳宝可梦', '海洋宝可梦', '铁铠宝可梦', '小鸽宝可梦', '天线宝可梦', '冷酷宝可梦', '蝎子宝可梦', '变身宝可梦', '拔刀宝可梦', '妖怪猫宝可梦', '皇家宝可梦', '慈爱宝可梦', '暴猿宝可梦', '装甲宝可梦', '毅力宝可梦', '蝌蚪宝可梦', '飞翅鱼宝可梦', '目光宝可梦', '古生代宝可梦', '幼马宝可梦', '时间宝可梦', '龙宝可梦', '食蚁兽宝可梦', '妖狐宝可梦', '眼珠宝可梦', '高温宝可梦', '蜡烛宝可梦', '意志宝可梦', '舞蹈宝可梦', '兔子宝可梦', '火马宝可梦', '花园宝可梦', '岩居宝可梦', '冬眠宝可梦', '老树宝可梦', '胜利宝可梦', '蛙宝可梦', '棉花糖宝可梦', '毒击宝可梦', '电子宝可梦', '谜宝可梦', '漂浮宝可梦', '尖尖宝可梦', '铁足宝可梦', '蛋宝可梦', '乐天宝可梦', '苹果汁宝可梦', '蛹宝可梦', '斑点熊猫宝可梦', '棉饰宝可梦', '美艳宝可梦', '粗鲁宝可梦', '虚拟宝可梦', '草羽宝可梦', '骨鹰宝可梦', '岩蛇宝可梦', '铠甲宝可梦', '妖精宝可梦', '球宝可梦', '拳斗宝可梦', '吐丝宝可梦', '火花宝可梦', '飞鼠宝可梦', '开朗宝可梦', '林蜥宝可梦', '泡蛙宝可梦', '磁铁宝可梦', '反派宝可梦', '封印宝可梦', '骏马宝可梦', '相随宝可梦', '传说宝可梦', '眼镜蛇宝可梦', '抓握宝可梦', '柔道宝可梦', '画皮宝可梦', '樱桃宝可梦', '异食宝可梦', '螺帽宝可梦', '火焰宝可梦', '长躯干宝可梦', '铜钟宝可梦', '紧咬宝可梦', '琵琶鱼宝可梦', '摘花宝可梦', '蛇宝可梦', '小鸟宝可梦', '威严宝可梦', '空间宝可梦', '奔雷宝可梦', '稻草人宝可梦', '悠游宝可梦']
    out_js = json_2_json(fn, trans_fun={
        # 'types': lambda j: [j[0]],
        'types': lambda j: list(set(sum([random.choices(types, k=5), [j[0]]], []))),
        'annos': lambda j: [j[0]],
        'input': lambda j: j[-1],
    }, ls_fun=lambda src: [j for j in sum([
        [[j[6], j[-7]] for j in pd.read_csv(fn).values.tolist()[1:]],
        [[j[6], j[-6]] for j in pd.read_csv(fn).values.tolist()[1:]],
        [[j[6], j[-5]] for j in pd.read_csv(fn).values.tolist()[1:]],
        [[j[6], j[-4]] for j in pd.read_csv(fn).values.tolist()[1:]],
        [[j[6], j[-3]] for j in pd.read_csv(fn).values.tolist()[1:]],
    ], []) if str(j[-1]) != 'nan'])
    types = list(set(sum([j['types'] for j in out_js], [])))
    



import os
import json
dir_path = 'E:/NLP/NER'

fns = os.listdir(dir_path)
type_cnt_map = {}
type_types_map = {}
for fn in fns:
    if 'aa.jsonl' not in fn:
        continue
    ls = open(f'{dir_path}/{fn}', 'r', encoding='utf-8').read().strip().split('\n')
    type = fn.split('_')[1]
    if type not in type_cnt_map:
        type_cnt_map[type] = 0
    type_cnt_map[type] += len(ls)
    for l in ls:
        j = json.loads(l)
        types = j['types']
        if type not in type_types_map:
            type_types_map[type] = set()
        if '' in types:
            print(fn, l)
        type_types_map[type].update(types)
print('数据量', type_cnt_map)
print('类别数', {k: len(list(type_types_map[k])) for k in type_types_map})



import requests
import json

def en2cn_google(text):
    return trans_google(text, 'en', 'zh-CN')

def trans_google(text, sl, tl):
    temp_url = 'https://translate.googleapis.com/translate_a/single?client=gtx&sl={sl}&tl={tl}&dt=t&q={q}'
    url = temp_url.format(q=text, sl=sl, tl=tl)
    result = requests.get(url)
    j = json.loads(result.content)
    try:
        cn = ''.join([i[0] for i in j[0]])
        return cn
    except:
        return ''

if __name__ == '__main__':
    # print(en2cn_google('hello world'))
    print(en2cn_google(''))



import re
import sys
sys.path.append('.')
sys.path.append('auto_anno')
from utils.format.txt_2_list import txt_2_list
from local_config import chat

gen_prompt = "你是一个有丰富数据的文本数据集，请帮我生成10句包含或属于以下类别的文本{类别}" \
    "\n输出格式参考：文本 | 类别" \
    "\n例如：这个东西真好 | 好评" \
    "\n{历史}" \
    "\n以下为10句类别为{类别}，且互不相关的句子："

def text_generate(type_arr, history=[]):
    type_arr.sort(key=lambda x: len(x), reverse=True) # 从长倒短排序，有限匹配长的，防止包含短的重复匹配，如：不好，好
    history_txt = ''.join([f'{q} | {a}\n' for q, a in history])
    user = gen_prompt
    user = user.replace('{类别}', str(type_arr)).replace('{历史}', history_txt)
    content = chat(user)
    print(f'---- text_generate ----\nuser {user}\ncontent {content}\n')
    # Check out in type_arr
    result = []
    content = re.sub(r'^\n\|', '|', content, flags=re.MULTILINE) # 防止标签换行
    content = content.replace('\n\n', '\n')
    ls = content.split('\n')
    for l in ls:
        _l = l
        has_type_arr = []
        if '|' in _l:
            _l = _l.split('|')[-1]
        for type in type_arr:
            if type in _l:
                _l = _l.replace(type, '')
                l = l.replace(f'({type})', '') # 从原文中删去类别在括号中的情况
                has_type_arr.append(type)
        # for type in has_type_arr:
        #     split_reg = '[\.,，。]'
        #     char_reg = '[^\.,，。]'
        #     l = re.sub(rf'{split_reg}{char_reg}*?{type}{char_reg}*?{split_reg}', '', l)
        l = l.replace('，属于', '|')
        l = l.split('|')[0].strip()
        l = re.sub('^[\d]+[\. 、\s]*', '', l)
        result.append([l, has_type_arr])

    return result


if __name__ == '__main__':
    # result = text_generate(['好评', '差评'])
    # result = text_generate(['有害', '无害']) # 这个类别不好生成，会有[但是]出现，同时有害无害定义不清晰
    # type_arr = txt_2_list('政治；经济；科技；文化；娱乐；民生；军事；教育；环保；其它')
    # result = text_generate(type_arr, history=[['前四个月我国外贸进出口同比增长 5.8%', ['经济']]])
    result = text_generate(['唐诗', '宋词'])
    # result = text_generate(['文明', '不文明'])
    print(result)



import re

def wash_tel(txt):
    _txt = ' ' + txt + ' '
    tels = re.findall(r'[^\d]\d{3}[\d-]{,8}\d{2}[^\d]', _txt)
    washed_txt = txt
    for tel in tels:
      tel = tel[1:-1]
      star = '*' * len(tel[3:-2])
      washed_txt = washed_txt.replace(tel, tel[:3] + star + tel[-2:])
    return washed_txt

def wash_idcard(txt):
    star = '*' * 13
    washed_txt = re.sub(r'(\d{3})(\d{13})([\dXx]{2})', rf'\1{star}\3', txt)
    return washed_txt

# 参考地址 https://www.jianshu.com/p/152e081fec1b
def q_2_b(uchar):
    """单个字符 全角转半角"""
    inside_code = ord(uchar)
    if inside_code == 0x3000:
        inside_code = 0x0020
    else:
        inside_code -= 0xfee0
    if inside_code < 0x0020 or inside_code > 0x7e: #转完之后不是半角字符返回原来的字符
        return uchar
    return chr(inside_code)

def wash_q_2_b(txt):
    """把字符串全角转半角"""
    return "".join([q_2_b(uchar) for uchar in txt])

if __name__ == '__main__':
    print(wash_tel('12345678901'))
    print(wash_tel('手机号：123-4567-8901'))
    print(wash_idcard('123456789012345678'))
    print(wash_idcard('我的身份证是33068119901214567X'))
    print(wash_tel('我的身份证是33068119901214567X'))
    print(wash_q_2_b('＂第　３　题，Ａ选项是对的．＂'))



import paddle
# paddlenlp==2.6.0rc0
# paddlepaddle-gpu==2.5.0

from paddlenlp.transformers import (
    ChatGLMConfig,
    ChatGLMForConditionalGeneration,
    ChatGLMTokenizer,
)

# 加载模型
#读取原始的chatglm-6b模型
model_name_or_path = 'THUDM/chatglm-6b'
tokenizer = ChatGLMTokenizer.from_pretrained(model_name_or_path)

config = ChatGLMConfig.from_pretrained(model_name_or_path)
paddle.set_default_dtype(config.paddle_dtype)

model = ChatGLMForConditionalGeneration.from_pretrained(
    model_name_or_path,
    tensor_parallel_degree=paddle.distributed.get_world_size(),
    tensor_parallel_rank=0,
    load_state_as_np=True,
    dtype=config.paddle_dtype,
)

model.eval()

# 函数定义，问glm问题（ask_glm）
# 输入参数：初始prompt, 最长输入长度，最长输出长度
def glm_single_QA(model,tokenizer,next_inputs,input_length,output_length):
    # 输入格式转换
    inputs = tokenizer(
        next_inputs,
        return_tensors="np",
        padding=True,
        max_length=input_length,
        truncation=True,
        truncation_side="left",
    )
    input_map = {}
    for key in inputs:
        input_map[key] = paddle.to_tensor(inputs[key])

    # 获取结果
    infer_result = model.generate(
        **input_map,
        decode_strategy="sampling",
        top_k=1,
        # top_p =5,
        max_length=output_length,
        use_cache=True,
        use_fast=True,
        use_fp16_decoding=True,
        repetition_penalty=1,
        # temperature = 0.95,
        temperature = 0.1,
        length_penalty=1,
    )[0]

    # 结果转换
    output = ''
    result = []
    for x in infer_result.tolist():
        res = tokenizer.decode(x, skip_special_tokens=True)
        res = res.strip("\n")
        result.append(res)
        output = output + res
    return output

def chat_chatglm_paddle(prompt, his=[], prompt_his_str='你：{}\n分身：{}'):
    Q_his = '\n'.join([prompt_his_str.format(i[0], i[1]) for i in his])
    if Q_his:
        Q_his += '\n'
    Q_motif = f'{Q_his}${prompt}'
    result=glm_single_QA(model,tokenizer,Q_motif,256,256)
    return result

def en2cn_glm(prompt):
    Q_motif = f'你是一个有百年经验的英汉翻译官，请你翻译以下句子\n{prompt}\n翻译结果为：'
    result=glm_single_QA(model,tokenizer,Q_motif,256,256)
    return result

if __name__ == '__main__':
  content = chat_chatglm_paddle('清华大学地址')
  print(content)



from setuptools import setup, find_packages


setup(
    name='auto_anno_2',
    version='0.1.4',
    packages=find_packages(),
    install_requires=[
        # list your project dependencies here
        'erniebot==0.3.1'
    ],
)

# python setup.py sdist bdist_wheel # 打包
# pip install dist/auto_anno_2-*-py3-none-any.whl # 本地安装，包名调整为打包出的包名
# twine upload dist/* # 上传到pypi
# pip install auto_anno_2 -U -i https://pypi.org/simple # 从pypi安装
# pip list|grep auto # 查看安装的包
# pip uninstall auto_anno_2 # 卸载包



from utils.anno.cv.traditional import traditional_detect
import flask
from flask import request
import flask_cors
import cv2
import numpy as np
import base64

app = flask.Flask(__name__)
# 允许跨域访问
flask_cors.CORS(app, supports_credentials=True)

@app.route('/detect', methods=['POST'])
def detect():
    data = {'errCode': 0}
    if flask.request.method == 'POST':
        # base64图片，json格式
        if flask.request.json.get('image'):
            image = flask.request.json['image']
            # base64转为cv2格式
            image = cv2.imdecode(np.fromstring(base64.b64decode(image), np.uint8), cv2.IMREAD_COLOR)
            # 转为cv2单通道格式
            image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
            height, width = image.shape

            result = traditional_detect(image)
            data['errCode'] = 0
            data['info'] = {
                'rect_angles': result.rect_angles,
                'contours': result.contours,
                'width': width,
                'height': height
            }
    return flask.jsonify(data)

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
